/*
 * GroupLens Common Utilities
 * Copyright © 2011 Regents of the University of Minnesota
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *  * Neither the name of the University of Minnesota nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software has been partly funded by NSF grant IIS 08-08692.
 */
package org.grouplens.common.dto;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;



/**
 * <p>
 * A Dto is a data transfer object intended for serialization and
 * deserialization of data beans. Subclasses of Dto represent their properties
 * as public fields. The field values are allowed to be null. All public fields
 * within the Dto are serialized/deserialized. To have finer control use the
 * {@link Exclude} annotation to control when a field is used.
 * </p>
 * <p>
 * A Dto subclasses should only use boxed primitives, other Dtos, or arrays of
 * those types as allowed types for their defined fields. This is to simplify
 * the implementation of the serialization engines. Generics are not supported.
 * </p>
 * <p>
 * The names of a Dto's properties match the field names within the class, and
 * is assume to be in camel-case. The name of a Dto is interpreted similarly,
 * except that the Dto suffix is removed: NamedTypeDto becomes 'namedType'. When
 * generating output or parsing input, content handlers are allowed to process
 * Dto names and properties to use property names that better match the content.
 * For example, the property 'propertyName' could be mapped to 'property-name'
 * or 'property_name'.
 * </p>
 * <p>
 * The default name generation assumes that the field or Dto name represents the
 * appropriate plurality. If the field is an array, it is assumed that the name
 * is plural, all other cases assume that the name is singular. A plural is
 * generated from a singular by adding an 's', and a singular is made by
 * reversing the process. Use the {@link Named} annotation for more flexibility.
 * </p>
 * </p>
 */
public abstract class Dto {
    /**
     * <p>
     * Annotation that specifies the plural and singular names for the Dto or
     * field when it is to be serialized or de-serialized. This overrides the
     * default name detection that strips the Dto suffix from the class name.
     * Name mapping as described above still occurs after checking this
     * annotation.
     * </p>
     * <p>
     * This annotation provides two variables for configuration. The two
     * properties should be used together to explicitly specify the singular and
     * plural names as necessary. When a singular name is needed (i.e. a simple
     * field or only one Dto), the singular() name is used. When a plural is
     * needed (i.e. an array or multiple Dtos), the plural() name is used.
     * </p>
     */
    @Retention(RetentionPolicy.RUNTIME)
    @Target( { ElementType.TYPE, ElementType.FIELD } )
    public @interface Named {
        public String singular();
        public String plural();
    }

    /**
     * By default, all public fields are serialized/deserialized. To limit when
     * a field is used, use this annotation on the field. If the annotation uses
     * IMPORT, the field will be excluded from imports. If the the annotation
     * has the value EXPORT the field will be excluded from exports.
     */
	@Retention(RetentionPolicy.RUNTIME)
	@Target( { ElementType.FIELD })
	public @interface Exclude {
	    public ExcludeType[] value();
	}
	
	public static enum ExcludeType {
	    IMPORT, EXPORT
	}
	
	/**
	 * By default, all of an object's fields are serialized as nested tag pairs
	 * in XML. This annotation is used to designate a field that should instead
	 * be serialized as an attribute included in the opening tag of the object.
	 * Note that only simple objects can be serialized as attributes, so this
	 * annotation will be ignored if it is applied to a Dto field.
	 */
	@Retention(RetentionPolicy.RUNTIME)
	@Target({ ElementType.FIELD })
	public @interface XmlAttribute {}
}
